function [ output_correct ] = dual_voted_correlation_1bit( rx_data, sample_rate_ref )
%DUAL_VOTED_CORRELATION_1BIT For testing a correlator with several mini voting correlators

        % Read the test vectors applied to the unit and calculate the expected
        % response. Also read the result vectors from the testbench, and plot
        % against the expected results.
        
        % The Barker code
        barker_11_1bit = [1 1 1 0 0 0 1 0 0 1 0];

        % Stuff for the multi corr
        sample_rate_mini = 3;           % Stuck at 2, due to clock constraints
        mini_corr_units = sample_rate_ref / sample_rate_mini;   % How many mini corrs
        pn_code_1bit_mini = reshape((barker_11_1bit' * ones(1, sample_rate_mini))', 1, length(barker_11_1bit)*sample_rate_mini);
        pn_code_1bit_mini = fliplr(pn_code_1bit_mini);

        % Downsample the received data into two registers, each holding unique data
        sub_corr_length = length(rx_data)/mini_corr_units;
        downsampled_rxdata = zeros(mini_corr_units, sub_corr_length);
        for j = 1:mini_corr_units
            downsampled_rxdata(j, 1:end) = rx_data(j : mini_corr_units : end);
        end
        % Sub correlation registers
        sub_corr = zeros(mini_corr_units, sub_corr_length);
        % Input registers for each sub correlator
        input_reg_mini = zeros(2,mini_corr_units,length(pn_code_1bit_mini));

        pn_code_small = fliplr(barker_11_1bit);
        % Run through every input sample
        for k = 1:sub_corr_length

            % For each sub correlator
            for j = 1:mini_corr_units
                % Shift in new data
                input_reg_mini(j, 2:end) = input_reg_mini(j, 1:end-1);
                input_reg_mini(j, 1) = downsampled_rxdata(j, k);

                % Vote on, and check each chip
                for l = 1:length(pn_code_small)
                    % Group together the samples, and vote.
                    chip_group = input_reg_mini(j, (l-1)*sample_rate_mini + 1 : l*sample_rate_mini);
                    voted_chip = mode(chip_group);

                    % Sum up the sub correlations
                    if (pn_code_small(l) == voted_chip)
                        sub_corr(j, k) = sub_corr(j, k) + 1;
                    end
                end
            end
        end
        % Up sample each mini-corr...
        mini_corr = zeros(mini_corr_units, sub_corr_length*mini_corr_units);
        for j = 1:mini_corr_units
            mini_corr(j, :) = reshape((sub_corr(j, 1:end)' * ones(1, mini_corr_units))', 1, sub_corr_length*mini_corr_units);
        end
        % ...shift them apart...
        for j = 2:mini_corr_units
            mini_corr(j, j:end) = mini_corr(j, 1:end-(j-1));
            mini_corr(j, 1:(j-1)) = 0;
        end
        % ...and add together
        output_correct = zeros(1, sub_corr_length*mini_corr_units);
        for j = 1:mini_corr_units
            output_correct = output_correct + mini_corr(j, :);
        end
end

