function [ output_test_multi ] = multi_correlator_1bit(rx_data, sample_rate_mini, mini_corr_units)
%MULTI_CORRELATOR_1BIT Summed output of several correlators out of phase

% Splits an input vector into M interleaving vectors, each split by a
% phase. The total correlation is the sum of the sub correlator outputs.
% This output correlation is identical to that of the standard correlator.

    % Stuff for the multi corr
    barker_11_1bit = [1 1 1 0 0 0 1 0 0 1 0];
    pn_code_1bit_mini = reshape((barker_11_1bit' * ones(1, sample_rate_mini))', 1, length(barker_11_1bit)*sample_rate_mini);
    pn_code_1bit_mini = fliplr(pn_code_1bit_mini);

    % Downsample the received data into two registers, each holding unique data
    sub_corr_length = length(rx_data)/mini_corr_units;
    downsampled_rxdata = zeros(mini_corr_units, sub_corr_length);
    for j = 1:mini_corr_units
        downsampled_rxdata(j, 1:end) = rx_data(j : mini_corr_units : end);
    end
    % Sub correlation registers
    sub_corr = zeros(mini_corr_units, sub_corr_length);
    % Input registers for each sub correlator
    input_reg_mini = randi(2,mini_corr_units,length(pn_code_1bit_mini));
    input_reg_mini = input_reg_mini - 1;

    % Run through every input sample
    for k = 1:sub_corr_length

        % For each sub correlator
        for j = 1:mini_corr_units
            % Shift in new data
            input_reg_mini(j, 2:end) = input_reg_mini(j, 1:end-1);
            input_reg_mini(j, 1) = downsampled_rxdata(j, k);
            % Regular correlation for each sub correlator
            for l = 1:length(pn_code_1bit_mini)
                if (pn_code_1bit_mini(l) == input_reg_mini(j, l))
                    sub_corr(j, k) = sub_corr(j, k) + 1;
                end
            end
        end
    end

    % Up sample each mini-corr...
    mini_corr = zeros(mini_corr_units, sub_corr_length*mini_corr_units);
    for j = 1:mini_corr_units
        mini_corr(j, :) = reshape((sub_corr(j, 1:end)' * ones(1, mini_corr_units))', 1, sub_corr_length*mini_corr_units);
    end
    % ...shift them apart...
    for j = 2:mini_corr_units
        mini_corr(j, j:end) = mini_corr(j, 1:end-(j-1));
        mini_corr(j, 1:(j-1)) = 0;
    end
    % ...and add together
    output_test_multi = zeros(1, sub_corr_length*mini_corr_units);
    for j = 1:mini_corr_units
        output_test_multi = output_test_multi + mini_corr(j, :);
    end
end

