function [ output_correct ] = voted_correlator_1bit( rx_data, sample_rate_ref )
%VOTED_CORRELATOR_1BIT For testing a voting correlator design

    % Read the test vectors applied to the unit and calculate the expected
    % response. Also read the result vectors from the testbench, and plot
    % against the expected results.

    % The Barker code
    barker_11_1bit = [1 1 1 0 0 0 1 0 0 1 0];

    % Voted for reduced resolution
    input_reg = zeros(1, length(barker_11_1bit)*sample_rate_ref);
    % Flip the PN code from right to left, so we xcorr and not convolute
    pn_code_small = fliplr(barker_11_1bit);
    output_correct = zeros(1, length(rx_data));
    for k = 1:length(rx_data)
        % Shift in new data
        input_reg(2:end) = input_reg(1:end-1);
        input_reg(1) = rx_data(k);

        % Each chip is oversampled by some factor. A voter is used to determine
        % whether the sample value to use for each chip, before it is
        % correlated. In the end, the final correlation of each chip is
        % checked. Reduces resolution greatly, but should still provide a
        % decent correlation estimate.

        % Vote on, and check each chip
        for l = 1:length(pn_code_small)
            % Group together the samples, and vote.
            chip_group = input_reg((l-1)*sample_rate_ref + 1 : l*sample_rate_ref);
            voted_chip = mode(chip_group);

            % Sum up the sub correlations
            if (pn_code_small(l) == voted_chip)
                output_correct(k) = output_correct(k) + 1;
            end
        end
    end
end

