% Reads an ADC data vector from a test bench file, and applies it to the
% mathematical model of both the reference design, and the voted design.
% Used to confirm that the voted design is sound.

%% Read test bench input file

% Define the test inputs file used
fref = fopen('rx_vect_for_tb.txt');
refData = textscan(fref,'%s','Delimiter','\n');
fclose(fref);
datalinesref = 4;  % Number of lines before the samples start

% Sample rate used
temp = refData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate = str2double(readsampleratestr{2});

if mod(sample_rate,2) == 0
   disp('Sampling rate is even! Use an odd sampling rate for the voted unit.')
   return
end

% Get the reference data used
reference_len = length(refData{1})-datalinesref;
rx_data = zeros(1, reference_len);
for i = 1:reference_len
    temp = refData{1}(i+datalinesref);
    rx_data(i) = str2double(temp{1});
end

% Get the data sequence to compare against
temp = refData{1}(1);
str_data = temp{1};
readDataStr = strsplit(str_data, 'Data sequence:');
str_data = readDataStr{2};
% Trim off the brackets
str_data = str_data(1:end);
str_data = str_data(3:end);
str_data = str_data(1:length(str_data)-1);
data = strread(str_data, '%d');

%% Cross correlate
output_voted = voted_correlator_1bit(rx_data, sample_rate);
output_standard = standard_correlation_1bit(rx_data, sample_rate);

%% Plot
% Expected data to receive
figure(1)
subplot(3, 1, 1);
%data_for_stem = zeros(1, (2 + length(data)));
%data_for_stem(2:1+length(data)) = data;
%data_for_stem(1) = 0;
data_for_stem = zeros(1, (1 + length(data)));
data_for_stem(1:length(data)) = data;
stem(data_for_stem);
title('Expected data sequence')

% Create threshold lines
upper_thres = 5 * sample_rate * ones(1, length(rx_data));
lower_thres = -5 * sample_rate * ones(1, length(rx_data));
upper_thres = (upper_thres + sample_rate * 11)/2;
lower_thres = (lower_thres + sample_rate * 11)/2;

subplot(3, 1, 2);
hold on;
plot(output_standard);
plot(upper_thres, 'r');
plot(lower_thres, 'r');
hold off;
title(['Cross-correlation standard, sample rate ' num2str(sample_rate) ' times chip frequency.'])
xlabel('Sample number')

% Create threshold lines
upper_thres = 11 * ones(1, length(rx_data)) - 1;
lower_thres = zeros(1, length(rx_data)) + 1;
 
subplot(3, 1, 3);
hold on;
plot(output_voted);
plot(upper_thres, 'r');
plot(lower_thres, 'r');
hold off;
title('With voting on over-sampled bits')
xlabel('Sample number')
