% For testing a voting correlator design
% Read the test vectors applied to the unit and calculate the expected
% response. Also read the result vectors from the testbench, and plot
% against the expected results.

% The Barker code
barker_11_1bit = [1 1 1 0 0 0 1 0 0 1 0];

% Define the test inputs file used
fref = fopen('rx_vect_for_tb.txt');
refData = textscan(fref,'%s','Delimiter','\n');
fclose(fref);
datalinesref = 4;  % Number of lines before the samples start

% Sample rate used
temp = refData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate_ref = str2double(readsampleratestr{2});

% Define the correlator output to check against
fid = fopen('corr_results.txt');
resData = textscan(fid,'%s','Delimiter','\n');
fclose(fid);
datalines = 3;  % Number of lines before the samples start

% Sample rate used
temp = resData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate_result = str2double(readsampleratestr{2});
% Check that the sample rate of the two files is the same
if (sample_rate_result ~= sample_rate_ref)
    display('Sample rates are not identical!')
end

% Translate array elements into the correct values
result_len = length(resData{1})-datalines;
results_vector = zeros(1, result_len);
for i = 1:result_len
    temp = resData{1}(i+datalines);
    results_vector(i) = str2double(temp{1});
end

% Get the reference data used
reference_len = length(refData{1})-datalinesref;
rx_data = zeros(1, reference_len);
for i = 1:reference_len
    temp = refData{1}(i+datalinesref);
    rx_data(i) = str2double(temp{1});
end

% Cross correlate
% Matched filter algorithm
pn_code_1bit = reshape((barker_11_1bit' * ones(1, sample_rate_ref))', 1, length(barker_11_1bit)*sample_rate_ref);
input_reg = zeros(1, length(pn_code_1bit));
% Flip the PN code from right to left, so we xcorr and not convolute
pn_code_1bit = fliplr(pn_code_1bit);
pn_code_small = fliplr(barker_11_1bit);
output_voted = zeros(1, length(rx_data));
errors = 0;
for k = 1:length(rx_data)
    % Shift in new data
    input_reg(2:end) = input_reg(1:end-1);
    input_reg(1) = rx_data(k);
    
    % Each chip is oversampled by some factor. A voter is used to determine
    % whether the sample value to use for each chip, before it is
    % correlated. In the end, the final correlation of each chip is
    % checked. Reduces resolution greatly, but should still provide a
    % decent correlation estimate.
    
    % Vote on, and check each chip
    for l = 1:length(pn_code_small)
        % Group together the samples, and vote.
        chip_group = input_reg((l-1)*sample_rate_ref + 1 : l*sample_rate_ref);
        voted_chip = mode(chip_group);
        
        % Sum up the sub correlations
        if (pn_code_small(l) == voted_chip)
            output_voted(k) = output_voted(k) + 1;
        end
    end
    
    % Count errors
    if (output_voted(k) ~= results_vector(k))
        errors = errors + 1;
    end
end

% Plot the result
figure(1)
%subplot(2, 1, 1);
hold on;
plot(output_voted);
plot(results_vector, 'or');
legend('Expected', 'Verilog');
hold off;
title(['Expected correlation VS Verilog waveform. Detected discrepancies: ' num2str(errors)])
xlabel('Sample number')
