% For testing a correlator with several mini voting correlators
% Read the test vectors applied to the unit and calculate the expected
% response. Also read the result vectors from the testbench, and plot
% against the expected results.

% The Barker code
barker_11_1bit = [1 1 1 0 0 0 1 0 0 1 0];

% Define the test inputs file used
fref = fopen('rx_vect_for_tb.txt');
refData = textscan(fref,'%s','Delimiter','\n');
fclose(fref);
datalinesref = 4;  % Number of lines before the samples start

% Sample rate used
temp = refData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate_ref = str2double(readsampleratestr{2});

% Define the correlator output to check against
fid = fopen('corr_results.txt');
resData = textscan(fid,'%s','Delimiter','\n');
fclose(fid);
datalines = 3;  % Number of lines before the samples start

% Sample rate used
temp = resData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate_result = str2double(readsampleratestr{2});
% Check that the sample rate of the two files is the same
if (sample_rate_result ~= sample_rate_ref)
    display('Sample rates are not identical!')
end

% Get the reference data used
reference_len = length(refData{1})-datalinesref;
rx_data = zeros(1, reference_len);
for i = 1:reference_len
    temp = refData{1}(i+datalinesref);
    rx_data(i) = str2double(temp{1});
end

% Translate array elements into the correct values
result_len = length(resData{1})-datalines;
results_vector = zeros(1, result_len);
%results_vector = zeros(1, result_len);
for i = 1:result_len
    temp = resData{1}(i+datalines);
    results_vector(i) = str2double(temp{1});
end

% Stuff for the multi corr
sample_rate_mini = 3;           % Stuck at 2, due to clock constraints
mini_corr_units = sample_rate_ref / sample_rate_mini;   % How many mini corrs
pn_code_1bit_mini = reshape((barker_11_1bit' * ones(1, sample_rate_mini))', 1, length(barker_11_1bit)*sample_rate_mini);
pn_code_1bit_mini = fliplr(pn_code_1bit_mini);

% Downsample the received data into two registers, each holding unique data
sub_corr_length = length(rx_data)/mini_corr_units;
downsampled_rxdata = zeros(mini_corr_units, sub_corr_length);
for j = 1:mini_corr_units
    downsampled_rxdata(j, 1:end) = rx_data(j : mini_corr_units : end);
end
% Sub correlation registers
sub_corr = zeros(mini_corr_units, sub_corr_length);
% Input registers for each sub correlator
input_reg_mini = zeros(2,mini_corr_units,length(pn_code_1bit_mini));

pn_code_small = fliplr(barker_11_1bit);
% Run through every input sample
for k = 1:sub_corr_length
    
    % For each sub correlator
    for j = 1:mini_corr_units
        % Shift in new data
        input_reg_mini(j, 2:end) = input_reg_mini(j, 1:end-1);
        input_reg_mini(j, 1) = downsampled_rxdata(j, k);
        
        % Vote on, and check each chip
        for l = 1:length(pn_code_small)
            % Group together the samples, and vote.
            chip_group = input_reg_mini(j, (l-1)*sample_rate_mini + 1 : l*sample_rate_mini);
            voted_chip = mode(chip_group);

            % Sum up the sub correlations
            if (pn_code_small(l) == voted_chip)
                sub_corr(j, k) = sub_corr(j, k) + 1;
            end
        end
    end
end
% Up sample each mini-corr...
mini_corr = zeros(mini_corr_units, sub_corr_length*mini_corr_units);
for j = 1:mini_corr_units
	mini_corr(j, :) = reshape((sub_corr(j, 1:end)' * ones(1, mini_corr_units))', 1, sub_corr_length*mini_corr_units);
end
% ...shift them apart...
for j = 2:mini_corr_units
    mini_corr(j, j:end) = mini_corr(j, 1:end-(j-1));
    mini_corr(j, 1:(j-1)) = 0;
end
% ...and add together
output_test_multi = zeros(1, sub_corr_length*mini_corr_units);
for j = 1:mini_corr_units
	output_test_multi = output_test_multi + mini_corr(j, :);
end

% Check for errors
errors = 0;
for k = 1:result_len
    % Count errors
    if (output_test_multi(k) ~= results_vector(k))
        errors = errors + 1;
    end
end



% Plot the result
figure(1)
%subplot(2, 1, 1);
hold on;
plot(output_test_multi);
plot(results_vector, 'or');
legend('Expected', 'Verilog');
hold off;
title(['Expected correlation VS Verilog waveform. Detected discrepancies: ' num2str(errors)])
xlabel('Sample number')
