//-----------------------------------------------------------------------------
//
// Title       : bit_cnter_TB
// Company     : Eirik Taylor
//
//-----------------------------------------------------------------------------
//
// Description : Test bench for a majority bit voters
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module bit_voter_TB ();
	
	localparam n = 5;	// Input bits 
	
	// Connections to the counter, outputs wire, inputs reg
	reg [n-1:0] currentBitVector;
	wire voted_res;
	
	// Some testbench variables	
	integer i, errors;
	reg [n-1:0] setbitsCorrect;
	reg correct_vote;
	
	// The device under test
	voter_unit_five DUT (
		.ballots(currentBitVector), 
		.vote(voted_res)
		);
	
	// Start up initializations
	initial
		begin 
			$display($time, " << Starting test bench >>");
			currentBitVector = (2**n) - 1;
			errors = 0;
			
			// Determine the correct answer
			setbitsCorrect = 0;	
			i = 0;
			for (i=0; i<n; i=i+1)	   
				begin
					if (currentBitVector[i] == 1) begin
							setbitsCorrect = setbitsCorrect + 1;
					end else begin
							setbitsCorrect = setbitsCorrect - 1;
						end
				end
			// Determine the correct vote based on the sign
			if (setbitsCorrect[n-1]) begin
					correct_vote = 0;
			end else begin
					correct_vote = 1;
				end
			
			//Verify the adder output
			#1 if (correct_vote != voted_res) begin
					$display(" Error for input: %b", currentBitVector);
					errors = errors + 1;
				end
		end
	
	// Increment the input vector every 10ns
	always
		#9 begin
			if (currentBitVector == 0) begin
					// Final vector has been tested!
					$display($time, " << Testing complete, %d erros detected! >>", errors);
					$finish;
			end else begin
					currentBitVector = currentBitVector - 1;
					
					// Determine the correct answer
					setbitsCorrect = 0;	
					i = 0;
					for (i=0; i<n; i=i+1)	   
						begin
							if (currentBitVector[i] == 1) begin
									setbitsCorrect = setbitsCorrect + 1;
							end else begin
									setbitsCorrect = setbitsCorrect - 1;
								end
						end
					// Determine the correct vote based on the sign
					if (setbitsCorrect[n-1]) begin
							correct_vote = 0;
					end else begin
							correct_vote = 1;
						end
					
					//Verify the adder output
					#1 if (correct_vote != voted_res) begin
							$display(" Error for input: %b", currentBitVector);
							errors = errors + 1;
						end
					
				end
		end
	
endmodule
