//-----------------------------------------------------------------------------
//
// Title       : seven_bit_cnter
// Design      : corr_counter_block
//
//-----------------------------------------------------------------------------
//
// Description : 7-bit parallel counter, using full-adders
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module seven_bit_cnter (in, sum);
	
	output [2:0] sum;
	input [6:0] in;	
	
	wire [1:0] parasum1, parasum2; 
	wire carry;
	
	// Three bit parallel counters	
	fulladder threebit1 (
		.A(in[0]), 
		.B(in[1]),
		.cin(in[2]),
		.s(parasum1[0]),
		.cout(parasum1[1])
		); 
	
	fulladder threebit2 (
		.A(in[3]), 
		.B(in[4]),
		.cin(in[5]),
		.s(parasum2[0]),
		.cout(parasum2[1])
		);
	
	// Full adders for the 2-bit ripple carry adder	
	fulladder rca_fa1 (
		.A(parasum1[1]), 
		.B(parasum2[1]),
		.cin(carry),
		.s(sum[1]),
		.cout(sum[2])
		);	
	
	fulladder rca_fa2 (
		.A(parasum1[0]), 
		.B(parasum2[0]),
		.cin(in[6]),
		.s(sum[0]),
		.cout(carry)
		);
	
endmodule
