% Applies a testbench ADC vector to the mathematical model of the two bit
% correlator

%% Read test bench input file

% Define the test inputs file used
fref = fopen('rx_vect_for_tb.txt');
refData = textscan(fref,'%s','Delimiter','\n');
fclose(fref);
datalinesref = 4;  % Number of lines before the samples start

% Sample rate used
temp = refData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate = str2double(readsampleratestr{2});

% Get the reference data used
reference_len = length(refData{1})-datalinesref;
rx_data = zeros(2, reference_len);
for i = 1:reference_len
    temp = refData{1}(i+datalinesref);
    value = temp{1};
    rx_data(1, i) = str2double(value(1));
    rx_data(2, i) = str2double(value(3));
end

% Get the data sequence to compare against
temp = refData{1}(1);
str_data = temp{1};
readDataStr = strsplit(str_data, 'Data sequence:');
str_data = readDataStr{2};
% Trim off the brackets
str_data = str_data(1:end);
str_data = str_data(3:end);
str_data = str_data(1:length(str_data)-1);
data = strread(str_data, '%d');

%% Correlate

% Using the offset binary model
output_correct = offsetbin_standard_correlation_2bit(rx_data, sample_rate);

% Cross correlation of radio data and PN code
% Format the rx vector, to contain +1, 0 and -1
merged_rx_data = rx_data(1, :) -rx_data(2, :);
PN_Code = [ 1 zeros(1,7) -1 zeros(1,6) 1 zeros(1,5) -1 zeros(1,4) 1 zeros(1,3) -1 zeros(1,2) 1 0 -1 ];
pn_code = reshape((PN_Code' * ones(1, sample_rate))', 1, length(PN_Code)*sample_rate);
r1_xy = xcorr(merged_rx_data, pn_code);
L = length(merged_rx_data) + length(pn_code) - 1;
start_corr = length(r1_xy) - L;
r1_xy = r1_xy(start_corr:end);
% Determine thresholds
thres_level = 4.8;
upper_thres = thres_level * sample_rate * ones(1, length(r1_xy));
lower_thres = -thres_level * sample_rate * ones(1, length(r1_xy));

% Voted model
output_voted = voted_correlation_2bit(rx_data, sample_rate);


%% Plot

figure(1)
% Expected data to receive
subplot(4, 1, 1);
data_for_stem = zeros(1, (2 + length(data)));
data_for_stem(1:length(data)) = data;
stem(data_for_stem);
title('Expected data sequence')

subplot(4, 1, 2);
hold on;
plot(r1_xy);
plot(upper_thres, 'r');
plot(lower_thres, 'r');
hold off;
title(['Cross Correlation of Received data and local PN code, sample rate ' num2str(sample_rate) ' times chip frequency.'])
xlabel('Sample number')

% Create threshold lines
midpoint = (16/2) * sample_rate;
thres = 4.4 * sample_rate;
upper_thres = (midpoint + thres) * ones(1, length(output_correct));
lower_thres = (midpoint - thres) * ones(1, length(output_correct));

subplot(4, 1, 3);
hold on;
plot(output_correct);
plot(upper_thres, 'r');
plot(lower_thres, 'r');
hold off;
title('Using the offset binary HDL model.')
xlabel('Sample number')

% Create threshold lines
midpoint = 0;
thres = 4.4;
upper_thres = (midpoint + thres) * ones(1, length(output_correct));
lower_thres = (midpoint - thres) * ones(1, length(output_correct));

subplot(4, 1, 4);
hold on;
plot(output_voted);
plot(upper_thres, 'r');
plot(lower_thres, 'r');
hold off;
title('Using the voted model.')
xlabel('Sample number')