function [ output ] = offsetbin_standard_correlation_2bit( rx_data, sample_rate_ref )
%OFFSETBIN_STANDARD_CORRELATION_2BIT Model of 2-bit correlator
    
    % Matched filter algorithm, provides full resolution output, in offset
    % binary form. Zeros are counted if the mirror register has a one in
    % it's position.
    
    %The actual PN code
    PN_Code = [ 1 zeros(1,7) -1 zeros(1,6) 1 zeros(1,5) -1 zeros(1,4) 1 zeros(1,3) -1 zeros(1,2) 1 0 -1 ];
    
    % The PN code, split into a positive and negative part
    PN_Code_pos = [ 1 zeros(1,7) 0 zeros(1,6) 1 zeros(1,5) 0 zeros(1,4) 1 zeros(1,3) 0 zeros(1,2) 1 0 0 ];
    PN_Code_neg = [ 0 zeros(1,7) 1 zeros(1,6) 0 zeros(1,5) 1 zeros(1,4) 0 zeros(1,3) 1 zeros(1,2) 0 0 1 ];
    
    % Expand the given PN code to match the sampling rate
    register_len = length(PN_Code_pos)*sample_rate_ref;
    pn_codepos_expanded = reshape((PN_Code_pos' * ones(1, sample_rate_ref))', 1, register_len);
    % Flip the PN code from right to left, so we xcorr and not convolute
    pn_codepos_expanded = fliplr(pn_codepos_expanded);
    
    pn_codeneg_expanded = reshape((PN_Code_neg' * ones(1, sample_rate_ref))', 1, register_len);
    pn_codeneg_expanded = fliplr(pn_codeneg_expanded);
    
    pn_code_expanded = reshape((PN_Code' * ones(1, sample_rate_ref))', 1, register_len);
    pn_code_expanded = fliplr(pn_code_expanded);
    
    
    % Shift in every sample, and check the correlation
    input_reg = zeros(2, register_len);
    output = zeros(1, length(rx_data));
    for k = 1:length(rx_data)
        % Shift in new data
        input_reg(:, 2:end) = input_reg(:, 1:end-1);
        input_reg(:, 1) = rx_data(:, k);

        % Correct bit counts 1, wrong bit counts 0
        for l = 1:register_len
            if (pn_code_expanded(l) ~= 0)
                if (pn_codepos_expanded(l) == input_reg(1, l))
                    output(k) = output(k) + 1;
                end

                if (pn_codeneg_expanded(l) == input_reg(2, l))
                    output(k) = output(k) + 1;
                end
            end
            
        end
    end
end

