function [ output ] = voted_correlation_2bit( rx_data, sample_rate_ref )
%VOTED_CORRELATION_2BIT Model of 2-bit correlator
    
    % Matched filter algorithm, votes on oversampled  bits.
    
    %The actual PN code
    PN_Code = [ 1 zeros(1,7) -1 zeros(1,6) 1 zeros(1,5) -1 zeros(1,4) 1 zeros(1,3) -1 zeros(1,2) 1 0 -1 ];
    PN_Code = fliplr(PN_Code);
    
    % Convert the received data into a single register
    rx_data_merged = rx_data(1, :) - rx_data(2, :);
    
    % Expand the given PN code to match the sampling rate
    register_len = length(PN_Code);
    
    % Shift in every sample, and check the correlation
    input_reg = zeros(1, register_len * sample_rate_ref);
    output = zeros(1, length(rx_data_merged));
    for k = 1:length(rx_data_merged)
        % Shift in new data
        input_reg(2:end) = input_reg(1:end-1);
        input_reg(1) = rx_data_merged(k);

        % Correct bit counts 1, wrong bit counts 0
        for l = 1:register_len
            if (PN_Code(l) ~= 0)
                % Create voted outputs
                chip_group = input_reg(1, (l-1)*sample_rate_ref + 1 : l*sample_rate_ref);
                voted_chip = mode(chip_group);
                
                if (PN_Code(l) == voted_chip)
                    output(k) = output(k) + 1;
                end
                
                if (-PN_Code(l) == voted_chip)
                    output(k) = output(k) - 1;
                end
            end
        end
    end
end