function [ detected_data ] = detect_correlator_peaks( correlation_vector, pn_code_len, upper_thres, lower_thres, sample_rate )
%DETECT_CORRELATOR_PEAKS Detects peaks corresponding to data bits.

% Find peaks in the correlator output above or below certain thresholds.
% Translate each into a data bit, before engaging a timeout.

    % The minimun expected time between peaks, given in samples
    TIME_OUT_THRES = floor( 0.7 * (sample_rate * pn_code_len));
    timeout = 0;
    % Create the data vector, will at most find a peak every time possible,
    % this vector can be shortened later by removing leftover zeros
    detected_data = zeros(1, ceil(length(correlation_vector) / TIME_OUT_THRES));
    data_index = 1;
    % Run through the correlation vector
    for j = 1:length(correlation_vector)
        % if the timeout is inactive, look for a peak
        if (timeout ~= 0)
            timeout = timeout - 1;
        else
            % Look for positive bit
            if (correlation_vector(j) >= upper_thres)
                detected_data(data_index) = 1;
                data_index = data_index + 1;
                timeout = TIME_OUT_THRES;

            % Look for negative bit
            elseif (correlation_vector(j) <= lower_thres)
                detected_data(data_index) = -1;
                data_index = data_index + 1;
                timeout = TIME_OUT_THRES;
            end
        end
    end

    % Trim zeros from the detected data vector
    zero_index = find(detected_data==0, 1, 'first');
    if (zero_index == 1)
        disp('No data detected!')
        detected_data = 0;
    else
        detected_data = detected_data(1:zero_index-1);
    end
end

