% This script reads the output file from a testbench and determines the
% correct type of correlator to compare against.

%%
% Define the test inputs file used
fref = fopen('rx_vect_for_tb.txt');
refData = textscan(fref,'%s','Delimiter','\n');
fclose(fref);
datalinesref = 4;  % Number of lines before the samples start

% Sample rate used
temp = refData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate_ref = str2double(readsampleratestr{2});

% Define the correlator output to check against
fid = fopen('corr_results.txt');
resData = textscan(fid,'%s','Delimiter','\n');
fclose(fid);
datalines = 4;  % Number of lines before the samples start

% Sample rate used
temp = resData{1}(3);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Sampling rate:');
sample_rate_result = str2double(readsampleratestr{2});
% Check that the sample rate of the two files is the same
if (sample_rate_result ~= sample_rate_ref)
    display('Sample rates are not identical!')
end
% Determine the correlator type to check against
temp = resData{1}(4);
str_sampling = temp{1};
readsampleratestr = strsplit(str_sampling, 'Correlator type:');
corr_type = str2double(readsampleratestr{2});

% Get the reference data used
reference_len = length(refData{1})-datalinesref;
rx_data = zeros(2, reference_len);
for i = 1:reference_len
    temp = refData{1}(i+datalinesref);
    value = temp{1};
    rx_data(1, i) = str2double(value(1));
    rx_data(2, i) = str2double(value(3));
end

% Translate array elements into the correct values
result_len = length(resData{1})-datalines;
%results_vector = zeros(1, result_len);
results_vector = zeros(1, result_len);
for i = 1:result_len
    temp = resData{1}(i+datalines);
    results_vector(i) = str2double(temp{1});
end

%%
% Pick a model to compare against
switch corr_type
   case 1
      %% Voted type
        output_correct = voted_correlator_1bit(rx_data, sample_rate_ref);

   case 2
      %% Dual voted type
        output_correct = dual_voted_correlation_1bit(rx_data, sample_rate_ref);
        
	case 4
      %% 2-bit Standard
        output_correct = offsetbin_standard_correlation_2bit(rx_data, sample_rate_ref);

    otherwise
	%% 1-bit Standard output
    output_correct = standard_correlation_1bit(rx_data, sample_rate_ref);
    
end

% Check for errors
errors = 0;
for k = 1:result_len
    % Count errors
    if (output_correct(k) ~= results_vector(k))
        errors = errors + 1;
    end
end

%%
% Plot the result
figure(1)
%subplot(2, 1, 1);
hold on;
plot(output_correct);
plot(results_vector, 'or');
legend('Expected', 'Verilog');
hold off;
title(['Expected correlation VS Verilog waveform. Detected discrepancies: ' num2str(errors)])
xlabel('Sample number')
