//-----------------------------------------------------------------------------
//
// Title       : eleven_bit_cnter_fa
// Design      : corr_counter_block
// Author      : 
// Company     : 
//
//-----------------------------------------------------------------------------
//
// Description : Counts the number of set bits in an 11-bit input vector, using
//					only full and half-adder cells.
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module eleven_bit_cnter_fa (in, sum);
	
	input [10:0] in;
	output [3:0] sum; 
	
	wire [3:0] sum;
	wire [2:0] sevensum;
	wire [1:0] threesum;
	wire [1:0] carry;
	
	// The seven bit parallel counter
	seven_bit_cnter sevenbitter (
		.in(in[6:0]),
		.sum(sevensum)
		);
	
	// The three bit parallel counter	
	fulladder threebitter (
		.A(in[7]), 
		.B(in[8]),
		.cin(in[9]),
		.s(threesum[0]),
		.cout(threesum[1])
		);	
	
	// Full adders for the 3-bit ripple carry adder	used to find the sum   
	halfadder rca_ha0 (
		.A(sevensum[2]), 
		.B(carry[1]),
		.s(sum[2]),
		.cout(sum[3])
		);
	
	fulladder rca_fa1 (
		.A(sevensum[1]), 
		.B(threesum[1]),
		.cin(carry[0]),
		.s(sum[1]),
		.cout(carry[1])
		);	
	
	fulladder rca_fa0 (
		.A(sevensum[0]), 
		.B(threesum[0]),
		.cin(in[10]),
		.s(sum[0]),
		.cout(carry[0])
		);
	
endmodule
