//-----------------------------------------------------------------------------
//
// Title       : four_bit_cnter
// Design      : corr_counter_block
//
//-----------------------------------------------------------------------------
//
// Description : 4-bit parallel counter
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module four_bit_cnter_ha (in, sum);
	
	output [2:0] sum;
	input [3:0] in;
	
	wire [1:0] partial_sum [0:1];
	wire rca_carry;
	
	halfadder rca_ha0 (
		.A(in[0]), 
		.B(in[1]),
		.s(partial_sum[0][0]),
		.cout(partial_sum[0][1])
		);	
	
	halfadder rca_ha1 (
		.A(in[2]), 
		.B(in[3]),
		.s(partial_sum[1][0]),
		.cout(partial_sum[1][1])
		);
	
	// Two bit ripple carry adder
	fulladder csa_fa0 (
		.A(partial_sum[1][1]), 
		.B(partial_sum[0][1]),
		.cin(rca_carry),
		.s(sum[1]),
		.cout(sum[2])
		);
	
	halfadder rca_ha2 (
		.A(partial_sum[1][0]), 
		.B(partial_sum[0][0]),
		.s(sum[0]),
		.cout(rca_carry)
		);
	
endmodule
