//-----------------------------------------------------------------------------
//
// Title       : correlator_tb
// Design      : parallel_correlator
// Author      : 
// Company     : 
//
//-----------------------------------------------------------------------------
//
// File        : c:\my_designs\parallel_correlator\parallel_correlator\src\correlator_tb.v
// Generated   : Tue Feb 11 13:15:01 2014
// From        : interface description file
// By          : Itf2Vhdl ver. 1.22
//
//-----------------------------------------------------------------------------
//
// Description : Test bench for a correlator using an 11-bit Barker sequence
//				Sampling rate can be set. Results are written out to a file,
//				where they can be plotted for easier verification.
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module correlator_tb ();
	
	// Correlator properties
	localparam samplerate = 3;
	localparam outputbits = 6;	
	
	// Define the type of correlator to compare against
	// 0 = standard, 1 = voted, 2 = dual voted, 4 = 2-bit standard
	localparam corr_type = 4;
	
	// Connections to the correlator, outputs wire, inputs reg
	wire [outputbits-1:0] correlator_result;
	reg reset;	
	reg clk;	
	reg ADC_sample_pos;
	reg ADC_sample_neg;
	reg adc_clock;	
	
	// Some testbench variables	
	integer adc_clock_divider;
	integer fd, results_file, count;
	reg [1023*8-1:0] dummy;
	
	// The correlator unit being tested
	hash_corr_three_muxed_alt DUT (
		.reset(reset), 
		.clk(clk),
		.sample_pos(ADC_sample_pos),
		.sample_neg(ADC_sample_neg),
		.corr(correlator_result),
		.adc_ready(adc_clock)
		);
	
	// Create a 50MHz clock
	always
		#10 clk = ~clk;
	
	// Create the ADC clock, which is tied to the global clock
	// Chip rate is 1MHz
	always @ (posedge clk)
		begin
			if (adc_clock_divider >= 50) begin
					adc_clock_divider = 0;
					adc_clock = 1'b1;
			end else begin
					adc_clock_divider = adc_clock_divider + 1;
					adc_clock = 1'b0;
				end
		end
	
	// Start-up, open file, reset system, and apply vectors
	initial
		begin
			fd = $fopen("rx_vect_for_tb.txt", "r");
			results_file = $fopen("corr_results.txt", "w");
			if (!fd) begin
					$display(" !! No test file found !!");
					$finish;
			end else begin
					// Move the file pointer up to the samples	
					count = $fgets(dummy, fd);
					count = $fgets(dummy, fd);
					count = $fgets(dummy, fd);
					count = $fgets(dummy, fd);
				end
			
			if (!results_file) begin
					$display(" !! Unable to create test results file !!");
					$finish;
			end else begin
					$fwrite(results_file, "Correlator outputs\nBit width: %d\nSampling rate: %d\nCorrelator type: %d\n", outputbits, samplerate, corr_type);
				end
			
			$display($time, " << Starting test bench >>");
			adc_clock_divider = 0;
			clk = 1'b0;
			reset = 1'b0;
			ADC_sample_pos = 1'b0;
			ADC_sample_neg = 1'b0;
			adc_clock = 1'b0;
			
			#20 reset = 1'b1;
			// All systems go! Start applying test vectors
			
			// Samples are loaded on each ADC clock, and the result checked
			// during the following ADC clock.
			@(posedge adc_clock);
			count = $fscanf(fd, "%b\t%b\n", ADC_sample_pos, ADC_sample_neg);
			
			while (!$feof(fd)) begin   
					@(posedge adc_clock);
					// Load new sample
					count = $fscanf(fd, "%b\t%b\n", ADC_sample_pos, ADC_sample_neg);
					// Write the correlator result
					$fwrite(results_file, "%d\n", correlator_result);
				end
			
			// Write the last correlator result
			@(posedge adc_clock);
			$fwrite(results_file, "%d\n", correlator_result);
			
			$fclose(fd);
			$fclose(results_file);
			$display($time, " << Testing complete! >>");
			$finish;
		end
	
endmodule
