//-----------------------------------------------------------------------------
//
// Title       : count_decode_neg
// Design      : hash_corr
//
//-----------------------------------------------------------------------------
//
// Description : Determines the two-bit correlation measure of a three samples,
//					based on a 5-bit hash. Used for negative hashes.
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module count_decode_neg (hash, count);
	
	output [2:0] count;
	input [3:0] hash;  
	
	reg [2:0] count;
	always @ (*)
		begin
			count = 3;
			case (hash)	
				4'b0000 : count = 3;
				4'b0001 : count = 2;
				4'b0010 : count = 2;
				4'b0011 : count = 1;
				4'b0100 : count = 2;
				4'b0101 : count = 1;
				4'b0110 : count = 1;
				4'b0111 : count = 0;
				4'b1000 : count = 3;
				4'b1001 : count = 3;
				4'b1010 : count = 3;	// Unused
				4'b1011 : count = 3;
				4'b1100 : count = 2;
				4'b1101 : count = 2;
				4'b1110 : count = 3;	// Unused
				4'b1111 : count = 4;
			endcase
		end
	
endmodule
