//-----------------------------------------------------------------------------
//
// Title       : count_decode_neg_alt
// Design      : hash_corr
//
//-----------------------------------------------------------------------------
//
// Description : Determines the two-bit correlation measure of a three samples,
//					based on a 5-bit hash. Used for negative hashes.
//
//				Alternate by not using a case selector, but boolean expressions
//				instead.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module count_decode_neg_alt (hash, count);
	
	output [2:0] count;
	input [3:0] hash;  
	
	wire not_a, not_b, not_c, not_d;
	wire a, b, c, d; 
	
	wire w1, w2, w3, w4, w5, w6;
	wire w7, w8, w9, w10;
	
	assign a = hash[3];	  
	assign b = hash[2];
	assign c = hash[1];
	assign d = hash[0];
	
	not(not_a, hash[3]); 
	not(not_b, hash[2]);
	not(not_c, hash[1]);
	not(not_d, hash[0]);
	
	// B2 = ABCD
	and(count[2], a,b,c,d);	
	
	// B1 = B'C' + B'D' + C'D' + AB' + AC' + AD'
	and(w1, not_b, not_c);
	and(w2, not_b, not_d);
	and(w3, not_c, not_d);
	and(w4, a, not_b);
	and(w5, a, not_c);
	and(w6, a, not_d);
	or(count[1], w1, w2, w3, w4, w5, w6);
	
	// B0 = B'C'D' + B'CD + A'BC'D + BCD' + AB'	
	and(w7, not_b, not_c, not_d);
	and(w8, not_b, c, d);
	and(w9, not_a, b, not_c, d);
	and(w10, b, c, not_d);
	or(count[0], w7, w8, w9, w10, w4);
	
endmodule
