//-----------------------------------------------------------------------------
//
// Title       : count_decoder
// Design      : hash_corr
//
//-----------------------------------------------------------------------------
//
// Description : Determines the two-bit correlation measure of a three samples,
//					based on a 5-bit hash. Counting is justified as 0 for negative
//					samples, and +2 for positive samples. A 0 where the is counted
//					as +1. If in negative polarity, -1 counts as +2, 0 as +1, and +1 as 0.
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module count_decoder (hash, polarity, count);
	
	output [2:0] count;
	input [4:0] hash;
	input polarity;
	
	reg [2:0] count;
	wire [2:0] count_pos, count_neg;
	
	// Connect the counter for positive sample groups
	count_decode_pos counterpos (
		.hash(hash[3:0]),
		.count(count_pos) 
		);
	
	// Connect the counter for negative sample groups
	count_decode_neg counterneg (
		.hash(hash[3:0]),
		.count(count_neg) 
		);
	
	// Select the correct output, depending on the polarity of the hash
	always @ (*)
		begin
			if (polarity) begin
					// Negative chip, check the sign of the hash
					if (hash[4]) begin
							count <= count_pos;
					end else begin
							count <= count_neg;
						end
					
			end else begin
					// Positive chip, check the sign of the hash
					if (hash[4]) begin
							count <= count_neg;
					end else begin
							count <= count_pos;
						end	 
				end
		end
	
endmodule
