//-----------------------------------------------------------------------------
//
// Title       : hash_gen
// Design      : hash_corr
//
//-----------------------------------------------------------------------------
//
// Description : Determines the new hash for a given old hash, depending on the
//				new sample received.
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module hash_gen (old_hash, adc_pos, adc_neg, new_hash);
	
	output [4:0] new_hash;
	input [4:0] old_hash;
	input adc_pos, adc_neg;
	
	reg [4:0] new_hash;
	wire [4:0] hash_one, hash_zero, hash_neg; 
	
	// Connect the hash generator for +1
	hasher_pos_one hasherpos (
		.old_hash(old_hash),
		.new_hash(hash_one) 
		);
	
	// Connect the hash generator for 0
	hasher_zero hasherzero (
		.old_hash(old_hash),
		.new_hash(hash_zero) 
		);
	
	// Connect the hash generator for -1
	hasher_neg_one hasherneg (
		.old_hash(old_hash),
		.new_hash(hash_neg) 
		);
	
	// Select the correct hash, depending on the sample received
	always @ (*)
		begin
			new_hash = 0;
			if (adc_pos) begin
					new_hash = hash_one;
			end else if (adc_neg) begin
					new_hash = hash_neg;
			end else begin
					new_hash = hash_zero;
				end
		end
	
endmodule
