//-----------------------------------------------------------------------------
//
// Title       : eight_bit_cnter
// Design      : corr_counter_block
// Author      : 
// Company     : 
//
//-----------------------------------------------------------------------------
//
// Description : Counts the number of set bits in a 8-bit input vector
//				 Uses two 3-bit counters, and a 2-bit counter. Carry-save
//				 adder used to sum the output.
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module eight_bit_cnter (in, sum);
	
	input [7:0] in;
	output [3:0] sum; 
	
	wire [3:0] sum;
	wire [1:0] partialsum [2:0];
	wire [1:0] csa_carry, csa_sum;
	wire rca_carry;
	
	// The 3- and 2-bit parallel counters
	fulladder threebitter1 (
		.A(in[0]),
		.B(in[1]),
		.cin(in[2]),
		.s(partialsum[0][0]),
		.cout(partialsum[0][1])
		);
	fulladder threebitter2 (
		.A(in[3]),
		.B(in[4]),
		.cin(in[5]),
		.s(partialsum[1][0]),
		.cout(partialsum[1][1])
		);
	halfadder twobitter (
		.A(in[6]),
		.B(in[7]),
		.s(partialsum[2][0]),
		.cout(partialsum[2][1])
		);
	
	// Full adders for the 2-bit carry save adder used to sum the five bit units
	fulladder csa_fa1 (
		.A(partialsum[2][1]), 
		.B(partialsum[1][1]),
		.cin(partialsum[0][1]),
		.s(csa_sum[1]),
		.cout(csa_carry[1])
		);	
	
	fulladder csa_fa0 (
		.A(partialsum[2][0]), 
		.B(partialsum[1][0]),
		.cin(partialsum[0][0]),
		.s(csa_sum[0]),
		.cout(csa_carry[0])
		);
	
	// Final adder used to sum the carry save adder output, and also add the final bit. csa_carry is shifted once before addition.
	// This leaves both the first and last adder cells open to replacement by half-adder cells.
	halfadder rca_fa2 (
		.A(rca_carry), 
		.B(csa_carry[1]),
		.s(sum[2]),
		.cout(sum[3])
		);
	
	halfadder rca_fa1 (
		.A(csa_sum[1]), 
		.B(csa_carry[0]),
		.s(sum[1]),
		.cout(rca_carry)
		);	
	
	assign sum[0] = csa_sum[0];
	
endmodule
