//-----------------------------------------------------------------------------
//
// Title       : sixteen_bit_cnter
// Design      : corr_counter_block
// Author      : 
// Company     : 
//
//-----------------------------------------------------------------------------
//
// Description : Counts the number of set bits in a 16-bit input vector
//				 Uses three 5-bit counters, and a final adder with carry in.
//
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module sixteen_bit_cnter (in, sum);
	
	input [15:0] in;
	output [4:0] sum; 
	
	wire [4:0] sum;
	wire [2:0] fivesum [2:0];
	wire [2:0] csa_carry, csa_sum;
	wire [2:0] rca_carry;
	
	// The five bit parallel counters
	five_bit_cnter fivebitter1 (
		.in(in[4:0]),
		.sum(fivesum[0])
		);
	five_bit_cnter fivebitter2 (
		.in(in[9:5]),
		.sum(fivesum[1])
		);
	five_bit_cnter fivebitter3 (
		.in(in[14:10]),
		.sum(fivesum[2])
		);
	
	// Full adders for the 3-bit carry save adder used to sum the five bit units
	fulladder csa_fa2 (
		.A(fivesum[2][2]), 
		.B(fivesum[1][2]),
		.cin(fivesum[0][2]),
		.s(csa_sum[2]),
		.cout(csa_carry[2])
		);
	
	fulladder csa_fa1 (
		.A(fivesum[2][1]), 
		.B(fivesum[1][1]),
		.cin(fivesum[0][1]),
		.s(csa_sum[1]),
		.cout(csa_carry[1])
		);	
	
	fulladder csa_fa0 (
		.A(fivesum[2][0]), 
		.B(fivesum[1][0]),
		.cin(fivesum[0][0]),
		.s(csa_sum[0]),
		.cout(csa_carry[0])
		);
	
	// Final adder used to sum the carry save adder output, and also add the final bit. csa_carry is shifted once before addition.
	// This leaves both the first and last adder cells open to replacement by half-adder cells.
	halfadder rca_fa3 (
		.A(rca_carry[2]), 
		.B(csa_carry[2]),
		.s(sum[3]),
		.cout(sum[4])
		);
	
	fulladder rca_fa2 (
		.A(csa_sum[2]), 
		.B(csa_carry[1]),
		.cin(rca_carry[1]),
		.s(sum[2]),
		.cout(rca_carry[2])
		);
	
	fulladder rca_fa1 (
		.A(csa_sum[1]), 
		.B(csa_carry[0]),
		.cin(rca_carry[0]),
		.s(sum[1]),
		.cout(rca_carry[1])
		);	
	
	halfadder rca_fa0 (
		.A(csa_sum[0]), 
		.B(in[15]),
		.s(sum[0]),
		.cout(rca_carry[0])
		);
	
endmodule
